@@ -71,7 +71,7 @@ TreeStyleTabWindowHelper.overrideExtensionsPreInit = function TSTWH_overrideExte
 										window.setTimeout(arguments.callee, 100);
 									return;
 								}
-								b.treeStyleTab.restoreStructure(tab);
+								b.treeStyleTab.handleRestoredTab(tab);
 							}, 0);
 						}).call(this, tab, id)
 					$1]]>
@@ -86,16 +86,20 @@ TreeStyleTabWindowHelper.overrideExtensionsPreInit = function TSTWH_overrideExte
 						(function() {
 							var tabsInfo = {};
 							var TST = TreeStyleTabService;
-							Array.slice(getBrowser().mTabContainer.childNodes)
-								.forEach(function(aTab) {
-									var index = this.getPermaTabLocalIndex(aTab);
-									if (index < 0) return;
-									var info = {};
-									TST.extraProperties.forEach(function(aProperty) {
-										info[aProperty] = TST.getTabValue(aTab, aProperty);
-									});
-									tabsInfo[this.permaTabs[index].id] = info;
-								}, this);
+							var allTabs = getBrowser().mTabContainer.childNodes;
+							for (let i = 0, maxi = allTabs.length; i < maxi; i++)
+							{
+								let tab = allTabs[i];
+								let index = this.getPermaTabLocalIndex(tab);
+								if (index < 0) continue;
+								let info = {};
+								for (let i = 0, maxi = TST.extraProperties.length; i < maxi; i++)
+								{
+									let property = TST.extraProperties[i];
+									info[property] = TST.getTabValue(tab, property);
+								}
+								tabsInfo[this.permaTabs[index].id] = info;
+							}
 							TST.setTreePref('permaTabsInfo', tabsInfo.toSource());
 						}).call(this);
 					]]>
@@ -117,16 +121,18 @@ TreeStyleTabWindowHelper.overrideExtensionsPreInit = function TSTWH_overrideExte
 		if ('getTabProperties' in sessionData && 'setTabProperties' in sessionData) {
 			let prefix = sv.kTMP_SESSION_DATA_PREFIX;
 			let sessionManager = window.TabmixSessionManager || window.SessionManager;
-			SessionData.tabTSTProperties = this.extraProperties.map(function(aProperty) {
+			sessionData.tabTSTProperties = this.extraProperties.map(function(aProperty) {
 				return prefix+aProperty;
 			});
 			eval('sessionData.getTabProperties = '+
 				sessionData.getTabProperties.toSource().replace(
 					'return tabProperties;',
 					<![CDATA[
-						this.tabTSTProperties.forEach(function(aProp) {
-							tabProperties += '|' + aProp + '=' + encodeURIComponent(aTab.getAttribute(aProp));
-						});
+						for (let i = 0, maxi = this.tabTSTProperties.length; i < maxi; i++)
+						{
+							let property = this.tabTSTProperties[i];
+							tabProperties += '|' + property + '=' + encodeURIComponent(aTab.getAttribute(property));
+						}
 					$&]]>
 				)
 			);
@@ -136,13 +142,15 @@ TreeStyleTabWindowHelper.overrideExtensionsPreInit = function TSTWH_overrideExte
 					<![CDATA[$&
 						var TSTProps = tabProperties.split('|');
 						tabProperties = TSTProps.shift();
-						TSTProps.forEach(function(aSet) {
-							var index = aSet.indexOf('=');
-							var name = aSet.substring(0, index);
-							var value = decodeURIComponent(aSet.substring(index+1));
+						for (let i = 0, maxi = TSTProps.length; i < maxi; i++)
+						{
+							let property = TSTProps[i];
+							let index = property.indexOf('=');
+							let name = property.substring(0, index);
+							let value = decodeURIComponent(property.substring(index+1));
 							if (name && value)
 								aTab.setAttribute(name, value);
-						});
+						}
 					]]>
 				)
 			);
@@ -151,7 +159,7 @@ TreeStyleTabWindowHelper.overrideExtensionsPreInit = function TSTWH_overrideExte
 					/(\}\))?$/,
 					<![CDATA[
 						if (gBrowser.treeStyleTab.useTMPSessionAPI)
-							gBrowser.treeStyleTab.restoreStructure(aTab);
+							gBrowser.treeStyleTab.handleRestoredTab(aTab);
 					$1]]>
 				)
 			);
@@ -160,7 +168,7 @@ TreeStyleTabWindowHelper.overrideExtensionsPreInit = function TSTWH_overrideExte
 				'return newTab;',
 				<![CDATA[
 					if (this.treeStyleTab.useTMPSessionAPI)
-						this.treeStyleTab.restoreStructure(newTab);
+						this.treeStyleTab.handleRestoredTab(newTab);
 				$&]]>
 			);
 			eval('tablib.init = '+source.join('gBrowser.restoreTab = '));
@@ -197,12 +205,13 @@ TreeStyleTabWindowHelper.overrideExtensionsPreInit = function TSTWH_overrideExte
 				'var tabcount = ',
 				<![CDATA[
 					gBrowser.treeStyleTab.collapseExpandAllSubtree(false, true);
-					gBrowser.treeStyleTab.getTabsArray(gBrowser)
-						.slice(1)
-						.reverse()
-						.forEach(function(aTab, aIndex) {
-							gBrowser.removeTab(aTab);
-						});
+					let (tabs = gBrowser.treeStyleTab.getTabsArray(gBrowser).slice(1).reverse()) {
+						for (let i = 0, maxi = tabs.length; i < maxi; i++)
+						{
+							let tab = tabs[i];
+							gBrowser.removeTab(tab);
+						}
+					}
 					TreeStyleTabService.restoringTree = true;
 				$&]]>
 			));
@@ -213,13 +222,17 @@ TreeStyleTabWindowHelper.overrideExtensionsPreInit = function TSTWH_overrideExte
 	// https://addons.mozilla.org/firefox/addon/4650
 	if ('FS_onFullerScreen' in window &&
 		sv.getTreePref('compatibility.FullerScreen')) {
-		'CheckIfFullScreen,FS_onFullerScreen,FS_onMouseMove'.split(',').forEach(function(aFunc) {
-			if (!(aFunc in window)) return;
-			eval('window.'+aFunc+' = '+window[aFunc].toSource().replace(
-				/FS_data.mTabs.(removeAttribute\("moz-collapsed"\)|setAttribute\("moz-collapsed", "true"\));/g,
-				'if (gBrowser.treeStyleTab.currentTabbarPosition == "top") { $& }'
-			));
-		}, this);
+		let (functions = 'CheckIfFullScreen,FS_onFullerScreen,FS_onMouseMove'.split(',')) {
+			for (let i = 0, maxi = functions.length; i < maxi; i++)
+			{
+				let func = functions[i];
+				if (!(func in window)) continue;
+				eval('window.'+func+' = '+window[func].toSource().replace(
+					/FS_data.mTabs.(removeAttribute\("moz-collapsed"\)|setAttribute\("moz-collapsed", "true"\));/g,
+					'if (gBrowser.treeStyleTab.currentTabbarPosition == "top") { $& }'
+				));
+			}
+		}
 	}
 
 	// TooManyTabs
@@ -233,7 +246,6 @@ TreeStyleTabWindowHelper.overrideExtensionsPreInit = function TSTWH_overrideExte
 	// https://addons.mozilla.org/firefox/addon/dragndrop-toolbars/
 	if ('globDndtb' in window &&
 		globDndtb.setTheStuff &&
-		sv.isGecko2 &&
 		sv.getTreePref('compatibility.DragNDropToolbars')) {
 		let reinitTabbar = function() {
 				TreeStyleTabService.stopRendering();
@@ -272,7 +284,6 @@ TreeStyleTabWindowHelper.overrideExtensionsPreInit = function TSTWH_overrideExte
 	if ('mtSidebarStartup' in window &&
 		'mtSidebarShutdown' in window &&
 		'mtPreventHiding' in window &&
-		sv.isGecko2 &&
 		sv.getTreePref('compatibility.OptimozTweaks')) {
 		eval('window.mtSidebarStartup = '+window.mtSidebarStartup.toSource().replace(
 			'{',
@@ -539,12 +550,18 @@ TreeStyleTabWindowHelper.overrideExtensionsAfterBrowserInit = function TSTWH_ove
 				{
 					switch (aEvent.type)
 					{
+						case 'TabOpen':
 						case 'TreeStyleTabAttached':
 						case 'TreeStyleTabParted':
 							var child = aEvent.originalTarget;
 							var parent = aEvent.parentTab;
 							if (child && parent) {
-								colorfulTabs.setColor(child, TreeStyleTabService.SessionStore.getTabValue(parent, 'tabClr'));
+								let color = TreeStyleTabService.SessionStore.getTabValue(parent, 'tabClr');
+								if (/^\d+,\d+,\d+$/.test(color))
+									color = 'rgb('+color+')';
+								window.setTimeout(function() {
+									colorfulTabs.setColor(child, color);
+								}, 0);
 							}
 							else if (child) {
 								TreeStyleTabService.SessionStore.setTabValue(child, 'tabClr', '');
@@ -556,6 +573,7 @@ TreeStyleTabWindowHelper.overrideExtensionsAfterBrowserInit = function TSTWH_ove
 							break;
 
 						case 'unload':
+							document.removeEventListener('TabOpen', this, false);
 							document.removeEventListener('TreeStyleTabAttached', this, false);
 							document.removeEventListener('TreeStyleTabParted', this, false);
 							document.removeEventListener('unload', this, false);
@@ -569,6 +587,7 @@ TreeStyleTabWindowHelper.overrideExtensionsAfterBrowserInit = function TSTWH_ove
 				'.display = ""'
 			)
 		);
+		document.addEventListener('TabOpen', listener, false);
 		document.addEventListener('TreeStyleTabAttached', listener, false);
 		document.addEventListener('TreeStyleTabParted', listener, false);
 		document.addEventListener('unload', listener, false);
@@ -668,25 +687,53 @@ TreeStyleTabWindowHelper.overrideExtensionsAfterBrowserInit = function TSTWH_ove
 	// Greasemonkey
 	// https://addons.mozilla.org/firefox/addon/748
 	if (sv.getTreePref('compatibility.Greasemonkey')) {
-		if ('GM_BrowserUI' in window && 'openInTab' in GM_BrowserUI) {
-			eval('GM_BrowserUI.openInTab = '+
-				GM_BrowserUI.openInTab.toSource().replace(
-					/(if\s*\(this\.isMyWindow\([^\)]+\)\)\s*\{\s*)(this\.tabBrowser)/,
-					'$1 TreeStyleTabService.readyToOpenChildTab($2); $2'
-				)
-			);
+		try {
+			let hitchModule = Components.utils.import('resource://greasemonkey/util/hitch.js', {});
+			let hitch = hitchModule.hitch;
+			if (hitch.toSource().indexOf('TreeStyleTabService') < 0) {
+				hitchModule.hitch = function(aObject, aMethod) {
+					if (typeof aMethod == 'function' &&
+						aMethod.toSource().indexOf('function openInTab') > -1) {
+						let originalOpenInTab = aMethod;
+						/**
+						 * This function must be replaced on scripts in "chrome:" URL, like this.
+						 * Otherwise the original openInTab() will raise violation error.
+						 * Don't move this hack into JS code modules with "resource:" URL.
+						 */
+						aMethod = function openInTab(aSafeContentWindow, aChromeWindow, aURL, aLoadInBackgtound) {
+							if (aChromeWindow.TreeStyleTabService)
+								aChromeWindow.TreeStyleTabService.readyToOpenChildTabNow(aSafeContentWindow);
+							return originalOpenInTab.apply(this, arguments);
+						};
+					}
+					return hitch.apply(this, arguments);
+				};
+			}
 		}
-		else if ('@greasemonkey.mozdev.org/greasemonkey-service;1' in Components.classes) {
-			let service = Components.classes['@greasemonkey.mozdev.org/greasemonkey-service;1'].getService().wrappedJSObject;
-			if (service) {
-				let _openInTab = service.__proto__._openInTab;
-				if (_openInTab.toSource().indexOf('TreeStyleTabService') < 0) {
-					service.__proto__._openInTab = function() {
-						let contentWindow = arguments[0];
-						let chromeWindow = arguments[1];
-						chromeWindow.TreeStyleTabService.readyToOpenChildTabNow(contentWindow);
-						return _openInTab.apply(this, arguments);
-					};
+		catch(e) {
+			dump(e+'\n');
+
+			// hacks for old versions
+			if ('GM_BrowserUI' in window && 'openInTab' in GM_BrowserUI) {
+				eval('GM_BrowserUI.openInTab = '+
+					GM_BrowserUI.openInTab.toSource().replace(
+						/(if\s*\(this\.isMyWindow\([^\)]+\)\)\s*\{\s*)(this\.tabBrowser)/,
+						'$1 TreeStyleTabService.readyToOpenChildTab($2); $2'
+					)
+				);
+			}
+			else if ('@greasemonkey.mozdev.org/greasemonkey-service;1' in Components.classes) {
+				let service = Components.classes['@greasemonkey.mozdev.org/greasemonkey-service;1'].getService().wrappedJSObject;
+				if (service && service.__proto__._openInTab) {
+					let _openInTab = service.__proto__._openInTab;
+					if (_openInTab.toSource().indexOf('TreeStyleTabService') < 0) {
+						service.__proto__._openInTab = function() {
+							let contentWindow = arguments[0];
+							let chromeWindow = arguments[1];
+							chromeWindow.TreeStyleTabService.readyToOpenChildTabNow(contentWindow);
+							return _openInTab.apply(this, arguments);
+						};
+					}
 				}
 			}
 		}
@@ -781,11 +828,13 @@ TreeStyleTabWindowHelper.overrideExtensionsAfterBrowserInit = function TSTWH_ove
 	if ('LinkyContext' in window &&
 		'prototype' in LinkyContext &&
 		sv.getTreePref('compatibility.Linky')) {
-		'doSelected,doSelectedText,doImages,doAll,doAllPics,doValidateAll,doValidateSelected'
-			.split(',').forEach(function(aMethod) {
-				if (!(aMethod in LinkyContext.prototype)) return;
-				eval('LinkyContext.prototype.'+aMethod+' = '+
-					LinkyContext.prototype[aMethod].toSource().replace(
+		let (methods = 'doSelected,doSelectedText,doImages,doAll,doAllPics,doValidateAll,doValidateSelected'.split(',')) {
+			for (let i = 0, maxi = methods.length; i < maxi; i++)
+			{
+				let method = methods[i];
+				if (!(method in LinkyContext.prototype)) continue;
+				eval('LinkyContext.prototype.'+method+' = '+
+					LinkyContext.prototype[method].toSource().replace(
 						'{',
 						'{ TreeStyleTabService.readyToOpenChildTab(null, true);'
 					).replace(
@@ -793,7 +842,8 @@ TreeStyleTabWindowHelper.overrideExtensionsAfterBrowserInit = function TSTWH_ove
 						'TreeStyleTabService.stopToOpenChildTab(); $1'
 					)
 				);
-			});
+			}
+		}
 	}
 
 	// QuickDrag
@@ -1288,16 +1338,19 @@ TreeStyleTabWindowHelper.overrideExtensionsDelayed = function TSTWH_overrideExte
 								break;
 
 							case sv.kEVENT_TYPE_BEFORE_TOOLBAR_CUSTOMIZATION:
-								tabbarToolboxes.forEach(function(aToolbox) {
-									aToolbox.removeAttribute('collapsed');
-								});
+								for (let i = 0, maxi = tabbarToolboxes.length; i < maxi; i++)
+								{
+									tabbarToolboxes[i].removeAttribute('collapsed');
+								}
 								break;
 
 							case sv.kEVENT_TYPE_AFTER_TOOLBAR_CUSTOMIZATION:
-								tabbarToolboxes.forEach(function(aToolbox) {
-									if (!aToolbox.firstChild.hasChildNodes())
-										aToolbox.setAttribute('collapsed', true);
-								});
+								for (let i = 0, maxi = tabbarToolboxes.length; i < maxi; i++)
+								{
+									let toolbox = tabbarToolboxes[i];
+									if (!toolbox.firstChild.hasChildNodes())
+										toolbox.setAttribute('collapsed', true);
+								}
 								break;
 
 							case 'unload':
@@ -1314,10 +1367,12 @@ TreeStyleTabWindowHelper.overrideExtensionsDelayed = function TSTWH_overrideExte
 			document.addEventListener(sv.kEVENT_TYPE_BEFORE_TOOLBAR_CUSTOMIZATION, listener, false);
 			document.addEventListener(sv.kEVENT_TYPE_AFTER_TOOLBAR_CUSTOMIZATION, listener, false);
 			document.addEventListener('unload', listener, false);
-			tabbarToolboxes.forEach(function(aToolbox) {
-				if (!aToolbox.firstChild.hasChildNodes())
-					aToolbox.setAttribute('collapsed', true);
-			});
+			for (let i = 0, maxi = tabbarToolboxes.length; i < maxi; i++)
+			{
+				let toolbox = tabbarToolboxes[i];
+				if (!toolbox.firstChild.hasChildNodes())
+					toolbox.setAttribute('collapsed', true);
+			}
 		}
 	}
 
