@@ -4,7 +4,7 @@
 pref("extensions.treestyletab.animation.enabled",           true);
 /**
  * Duration of animations.
- * DO NOT CHANGE THE PREFS ON FIREFOX 4 OR LATER, because actual duration of
+ * DO NOT CHANGE THE PREFS, because actual duration of
  * animations are defined in CSS. (CSS Transitions)
  * These prefs are used by the JavaScript implementation.
  */
@@ -13,7 +13,7 @@ pref("extensions.treestyletab.animation.collapse.duration", 150);
 
 /**
  * Size of resizable tab bar. They are completely ignored if "Tabs on Top"
- * is activated on Firefox 4 or later.
+ * is activated.
  */
 pref("extensions.treestyletab.tabbar.width",           200);
 pref("extensions.treestyletab.tabbar.height",          32);
@@ -57,8 +57,7 @@ pref("extensions.treestyletab.tabbar.scroll.duration", 250);
 pref("extensions.treestyletab.tabbar.scrollToNewTab.mode", 1);
 /**
  * Scrollbar in the leftside vertical tab bar should be shown leftmost position
- * instead of between tabs and the content area. This option works only for
- * Firefox 4 or later.
+ * instead of between tabs and the content area.
  */
 pref("extensions.treestyletab.tabbar.invertScrollbar", true);
 
@@ -69,6 +68,16 @@ pref("extensions.treestyletab.tabbar.invertScrollbar", true);
 pref("extensions.treestyletab.tabbar.narrowScrollbar", true);
 pref("extensions.treestyletab.tabbar.narrowScrollbar.size", "10px");
 
+/**
+ * On some environments (ex. GNOME3 on Linux), "narrow scrollbar" cannot get
+ * narrow width because system appearance possibly expand the size of scrollbar
+ * elements. To prevent this issue, we have to override "-moz-appeearance" defined
+ * by Firefox's default theme.
+ */
+// pref("extensions.treestyletab.tabbar.narrowScrollbar.overrideSystemAppearance", false);
+pref("extensions.treestyletab.platform.default.tabbar.narrowScrollbar.overrideSystemAppearance", false);
+pref("extensions.treestyletab.platform.Linux.tabbar.narrowScrollbar.overrideSystemAppearance", true);
+
 /**
  * The skin of the tab bar. Available styles are:
  *  "default", "flat", "mixed", "vertigo", "metal" and "sidebar".
@@ -105,6 +114,15 @@ pref("extensions.treestyletab.tabbar.autoHide.mode.fullscreen",        1);
  */
 pref("extensions.treestyletab.tabbar.autoHide.mode.toggle",            2);
 pref("extensions.treestyletab.tabbar.autoHide.mode.toggle.fullscreen", 1);
+/**
+ * When the tab bar is shown, content area must receive mousemove events
+ * to hide the tab bar. However, rectangle drawn by plugins (like <embed>)
+ * doesn't fire mousemove event even if mouse pointer moves on it. If this
+ * preference is "true", TST showns transparent screen on the content area
+ * to receive mousemove events anyway. If the screen is annoying for you,
+ * (actually it prevents click events on the content area)
+ */
+pref("extensions.treestyletab.tabbar.autoHide.contentAreaScreen.enabled", true);
 /**
  * Triggers for the "Auto Hide" feature. They can be controlled via the
  * configuration dialog.
@@ -188,6 +206,14 @@ pref("extensions.treestyletab.stackCollapsedTabs", true);
  */
 pref("extensions.treestyletab.showBorderForFirstTab",  false);
 
+/**
+ * The role of the "counter" in each tab.
+ *  1 = Indicate number of all tabs in the tree (including the parent tab itself)
+ *  2 = Indicate number of contained tabs in the collapsed tree (imitating file managers)
+ */
+pref("extensions.treestyletab.counter.role.horizontal", 1);
+pref("extensions.treestyletab.counter.role.vertical", 2);
+
 /**
  * Activates "auto-expand/collapse of tabs while dragging".
  * When you're dragging something, a collapsed tree will be expanded
@@ -301,11 +327,22 @@ pref("extensions.treestyletab.twisty.expandSensitiveArea", true);
 pref("extensions.treestyletab.clickOnIndentSpaces.enabled", true);
 
 /**
- * If this is "true", TST shows all titles of tabs in the tree by the tooltip
- * for collapsed tree. If this is "false", or the tree is not collapsed,
- * tooltip will say only the title of the single tab.
+ * Contents of the tooltip on tabs.
+ * 0 = Firefox default (show the title of the one tab)
+ * 1 = show all titles of tabs in the tree only for collapsed tree
+ * 2 = show all titles of tabs in the tree always
  */
-pref("extensions.treestyletab.tooltip.includeChildren",  true);
+pref("extensions.treestyletab.tooltip.mode", 2);
+/**
+ * Maximum count of tabs in a tooltip. Tabs over this limit are
+ * not shown in the tooltip.
+ */
+pref("extensions.treestyletab.tooltip.maxCount", 10);
+/**
+ * After this delay, TST shows "full tooltip" to show whole tree.
+ * Negative value means "do not show full tooltip".
+ */
+pref("extensions.treestyletab.tooltip.fullTooltipDelay", 2000);
 
 /**
  * Visibility of extra menu items for the context menu on tabs, inserted by TST.
@@ -385,14 +422,18 @@ pref("extensions.treestyletab.autoCollapseExpandSubtreeOnSelect",      true);
 pref("extensions.treestyletab.autoCollapseExpandSubtreeOnSelect.onCurrentTabRemove", true);
 /**
  * When you press Ctrl-Tab/Shift-Ctrl-Tab, Firefox advances focus of tabs.
- * If this is "true", TST doesn't collapse/expand trees while you are switching
+ * If this is "true", TST expands focused tree after a delay specified
+ * by another preference "autoExpandSubtreeOnSelect.whileFocusMovingByShortcut.delay".
+ * (If the delay is "0", TST dynamiclaly expands focused tree even if
+ * you are browsing tabs by those shortcuts.)
+ * If this is "false", TST doesn't expand trees while you are switching
  * tab focus by those keyboard shortcuts. And then, after you release the Ctrl
  * key, TST will expand the tree if the newly selected tab have its collapsed
  * tree.
- * If this is "false", TST dynamiclaly collapses/expands focused tree even if
- * you are browsing tabs by those shortcuts.
  */
-pref("extensions.treestyletab.autoCollapseExpandSubtreeOnSelect.whileFocusMovingByShortcut", false);
+pref("extensions.treestyletab.autoExpandSubtreeOnSelect.whileFocusMovingByShortcut", true);
+pref("extensions.treestyletab.autoExpandSubtreeOnSelect.whileFocusMovingByShortcut.collapseOthers", true);
+pref("extensions.treestyletab.autoExpandSubtreeOnSelect.whileFocusMovingByShortcut.delay", 800);
 /**
  * If this is "true", TST expands/collapses tree by double-clicking on tabs.
  * Otherwise TST simply ignores such actions.
@@ -450,8 +491,6 @@ pref("extensions.treestyletab.autoAttach.newTabCommand", 0);
 /**
  * How to treat new tabs from middle click (or Ctrl-click) on the "New Tab" button.
  *  Options are same to extensions.treestyletab.autoAttach.newTabCommand.
- * NOTE: This pref affects only on Firefox 4 or later. Firefox 3.6 doesn't open
- *       new blank tabs by middle click on the "New Tab" button.
  */
 pref("extensions.treestyletab.autoAttach.newTabButton", 1);
 /**
@@ -470,7 +509,7 @@ pref("extensions.treestyletab.autoAttach.goButton", 1);
  *  0 = Focus to both visible and collapsed tabs. (If a collapsed tab is
  *      focused, the tree will be expanded by another pref "autoExpandSubtreeOnCollapsedChildFocused".
  *  1 = Focus to visible tabs. Collapsed tabs will be skipped. (But if the tree 
- *      is expanded by "autoCollapseExpandSubtreeOnSelect.whileFocusMovingByShortcut",
+ *      is expanded by "autoExpandSubtreeOnSelect.whileFocusMovingByShortcut",
  *      visible tabs in the tree can be focused.)
  */
 pref("extensions.treestyletab.focusMode", 1);
@@ -519,7 +558,7 @@ pref("extensions.treestyletab.closeRootBehavior", 3);
 pref("extensions.treestyletab.undoCloseTabSet.behavior", 3);
 
 /**
- * On Firefox 4 or later, status panel possibly covers the tab bar. If "true",
+ * Status panel possibly covers the tab bar. If "true",
  * TST repositions (and resizes) the status panel automatically.
  * For compatibility, you can set this to "false". Then TST doesn't controll
  * the status panel.
@@ -527,12 +566,16 @@ pref("extensions.treestyletab.undoCloseTabSet.behavior", 3);
 pref("extensions.treestyletab.repositionStatusPanel", true);
 
 /**
- * EXPERIMENTAL, DON'T SET THIS "true" !!
- * If this is "true", TST restores tree structure before SSTabRestoring events
- * are fired. Otherwise TST restores tree structure based on SSTabRestoring
- * events.
+ * On Firefox 8 or later, TST can restore tree structure before SSTabRestoring
+ * events. SSTabRestoring based restore can break tree for duplicated tabs via
+ * the SessionStore service, so, "1" and "2" are recommended.
+ * ("2" will take much time on the startup if there are too many background groups.)
+ *  0 = Restore trees on SSTabRestoring. (disable "fast restore")
+ *  1 = Restore trees before SSTabRestoring.
+ *      Trees in background groups are restored when they are shown.
+ *  2 = Restore all trees before SSTabRestoring including background groups.
  */
-pref("extensions.treestyletab.restoreTreeOnStartup", false);
+pref("extensions.treestyletab.restoreTree.level", 1);
 
 /**
  * TST overrides some internal prefs of Firefox itself, because they can
@@ -542,7 +585,6 @@ pref("browser.link.open_newwindow.restriction.override", 0);
 pref("browser.tabs.loadFolderAndReplace.override", false);
 pref("browser.tabs.insertRelatedAfterCurrent.override", false);
 pref("browser.tabs.insertRelatedAfterCurrent.override.force", true);
-pref("extensions.treestyletab.tabsOnTopShouldBeRestored", false);
 
 /**
  * Extra commands for selected tabs (Multiple Tab Handler)
@@ -558,10 +600,10 @@ pref("extensions.treestyletab.createSubtree.underParent", true);
 
 /**
  * Size of pinned tabs in the vertical tab bar.
- * "width" can accept a special value "-1" which means "expand to the width of the tab bar".
+ * If true, they will be faviconized. Otherwise, they expand to the width
+ * of the tab bar.
  */
-pref("extensions.treestyletab.pinnedTab.width", 24);
-pref("extensions.treestyletab.pinnedTab.height", 24);
+pref("extensions.treestyletab.pinnedTab.faviconized", true);
 
 /**
  * Compatibility hack flags for other addons. They can be disabled by each
@@ -599,6 +641,7 @@ pref("extensions.treestyletab.compatibility.PersonalTitlebar", true);
 pref("extensions.treestyletab.compatibility.QuickDrag", true);
 pref("extensions.treestyletab.compatibility.RemoveNewTabButton", true);
 pref("extensions.treestyletab.compatibility.SBMCounter", true);
+pref("extensions.treestyletab.compatibility.Scriptish", false);
 pref("extensions.treestyletab.compatibility.SelectionLinks", true);
 pref("extensions.treestyletab.compatibility.SessionManager", true);
 pref("extensions.treestyletab.compatibility.SmoothlyCloseTabs", true);
