@@ -14,10 +14,10 @@
  * The Original Code is the Tree Style Tab.
  *
  * The Initial Developer of the Original Code is SHIMODA Hiroshi.
- * Portions created by the Initial Developer are Copyright (C) 2010-2011
+ * Portions created by the Initial Developer are Copyright (C) 2010-2012
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): SHIMODA Hiroshi <piro@p.club.ne.jp>
+ * Contributor(s): SHIMODA Hiroshi <piro.outsider.reflex@gmail.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -40,8 +40,6 @@ const Ci = Components.interfaces;
  
 Components.utils.import('resource://gre/modules/XPCOMUtils.jsm'); 
 
-// Components.utils.import('resource://treestyletab-modules/rap.js');
-// rap();
 Components.utils.import('resource://treestyletab-modules/lib/prefs.js');
 Components.utils.import('resource://treestyletab-modules/lib/namespace.jsm');
 var window = getNamespaceFor('piro.sakura.ne.jp');
@@ -76,12 +74,11 @@ XPCOMUtils.defineLazyGetter(this, 'autoScroll', function() {
 	Components.utils.import('resource://treestyletab-modules/lib/autoScroll.js', {});
 	return window['piro.sakura.ne.jp'].autoScroll;
 });
-XPCOMUtils.defineLazyGetter(this, 'confirmWithTab', function() {
+XPCOMUtils.defineLazyGetter(this, 'confirmWithPopup', function() {
 	var ns = {};
-	Components.utils.import('resource://treestyletab-modules/lib/confirmWithTab.js', ns);
-	return ns.confirmWithTab;
+	Components.utils.import('resource://treestyletab-modules/lib/confirmWithPopup.js', ns);
+	return ns.confirmWithPopup;
 });
-// rap('end of definition of lazy getters');
  
 var TreeStyleTabUtils = { 
 	__proto__ : window['piro.sakura.ne.jp'].prefs,
@@ -97,12 +94,12 @@ var TreeStyleTabUtils = {
 	kINSERT_BEFORE      : 'treestyletab-insert-before',
 	kINSERT_AFTER       : 'treestyletab-insert-after',
 	kCLOSED_SET_ID      : 'treestyletab-closed-set-id',
-	kSTRUCTURE          : 'treestyletab-tree-structure',
 
 	kID_RESTORING       : 'treestyletab-id-restoring',
 	kCHILDREN_RESTORING : 'treestyletab-children-restoring',
 
 	kSUBTREE_COLLAPSED  : 'treestyletab-subtree-collapsed',
+	kSUBTREE_EXPANDED_MANUALLY : 'treestyletab-subtree-expanded-manually',
 	kCOLLAPSED          : 'treestyletab-collapsed',
 	kCOLLAPSED_DONE     : 'treestyletab-collapsed-done',
 	kCOLLAPSING_PHASE   : 'treestyletab-collapsing-phase',
@@ -119,7 +116,6 @@ var TreeStyleTabUtils = {
 	kMODE               : 'treestyletab-mode',
 
 	kHIDE_NEWTAB        : 'treestyletab-hide-newtab-button',
-	kHIDE_ALLTABS       : 'treestyletab-hide-alltabs-button', /* legacy feature for Firefox 3.6 or olders */
 	kSTYLE              : 'treestyletab-style',
 	kFIRSTTAB_BORDER    : 'treestyletab-firsttab-border',
 	kFIXED              : 'treestyletab-tabbar-fixed',
@@ -131,6 +127,7 @@ var TreeStyleTabUtils = {
 	kINVERT_SCROLLBAR   : 'treestyletab-invert-scrollbar',
 	kNARROW_SCROLLBAR   : 'treestyletab-narrow-scrollbar',
 	kFAVICONIZED        : 'treestyletab-faviconized',
+	kBG_NOTIFY_PHASE    : 'treestyletab-notifybgtab-phase',
 
 	kTAB_INVERTED          : 'treestyletab-tab-inverted',
 	kTAB_CONTENTS_INVERTED : 'treestyletab-tab-contents-inverted',
@@ -147,9 +144,6 @@ var TreeStyleTabUtils = {
  
 /* classes */ 
 	kTWISTY                     : 'treestyletab-twisty',
-	kTWISTY_CONTAINER           : 'treestyletab-twisty-container',
-	kDROP_MARKER                : 'treestyletab-drop-marker',
-	kDROP_MARKER_CONTAINER      : 'treestyletab-drop-marker-container',
 	kCOUNTER                    : 'treestyletab-counter',
 	kCOUNTER_CONTAINER          : 'treestyletab-counter-container',
 	kCOUNTER_PAREN              : 'treestyletab-counter-paren',
@@ -244,6 +238,13 @@ var TreeStyleTabUtils = {
 	kCLOSE_PARENT_BEHAVIOR_SIMPLY_DETACH_ALL_CHILDREN : 4,
 	kCLOSE_PARENT_BEHAVIOR_CLOSE_ALL_CHILDREN         : 2, // onTabRemoved only
 
+	kRESTORE_TREE_LEVEL_NONE   : 0,
+	kRESTORE_TREE_ONLY_VISIBLE : 1,
+	kRESTORE_TREE_ALL          : 2,
+
+	kCOUNTER_ROLE_ALL_TABS       : 1,
+	kCOUNTER_ROLE_CONTAINED_TABS : 2,
+
 	MAX_TABBAR_SIZE_RATIO        : 0.8,
 	DEFAULT_SHRUNKEN_WIDTH_RATIO : 0.67,
  
@@ -261,6 +262,9 @@ var TreeStyleTabUtils = {
 	shouldExpandTwistyArea : true,
 
 	scrollToNewTabMode : false,
+
+	counterRoleHorizontal : -1,
+	counterRoleVertical : -1,
  
 	get SessionStore() { 
 		if (!this._SessionStore) {
@@ -303,6 +307,15 @@ var TreeStyleTabUtils = {
 	},
 	_PromptService : null,
 
+	get FocusManager()
+	{
+		if (!this._FocusManager) {
+			this._FocusManager = Cc['@mozilla.org/focus-manager;1'].getService(Ci.nsIFocusManager);
+		}
+		return this._FocusManager;
+	},
+	 _FocusManager : null,
+
 	get XULAppInfo() {
 		if (!this._XULAppInfo) {
 			this._XULAppInfo = Cc['@mozilla.org/xre/app-info;1'].getService(Ci.nsIXULAppInfo).QueryInterface(Ci.nsIXULRuntime);
@@ -318,9 +331,9 @@ var TreeStyleTabUtils = {
 	},
 	_Comparator : null,
  
-	get isGecko2() 
+	get isGecko10OrLater() 
 	{
-		return this.Comparator.compare(this.XULAppInfo.version, '4.0b5') > 0;
+		return this.Comparator.compare(this.XULAppInfo.version, '10.0a') > 0;
 	},
  
 	get treeBundle() { 
@@ -337,12 +350,12 @@ var TreeStyleTabUtils = {
  
 	init : function TSTUtils_init() 
 	{
-// rap('utils/init start');
 		if (this._initialized) return;
 
 		this.isMac = this.XULAppInfo.OS == 'Darwin';
 
 		this.applyPlatformDefaultPrefs();
+		this.migratePrefs();
 
 		this.addPrefListener(this);
 
@@ -359,6 +372,8 @@ var TreeStyleTabUtils = {
 		this.onPrefChange('extensions.treestyletab.animation.indent.duration');
 		this.onPrefChange('extensions.treestyletab.animation.collapse.duration');
 		this.onPrefChange('extensions.treestyletab.twisty.expandSensitiveArea');
+		this.onPrefChange('extensions.treestyletab.counter.role.horizontal');
+		this.onPrefChange('extensions.treestyletab.counter.role.vertical');
 
 		try {
 			if (this.XULAppInfo.OS == 'WINNT')
@@ -367,23 +382,153 @@ var TreeStyleTabUtils = {
 		catch(e) {
 			dump(e+'\n');
 		}
-// rap('utils/init end');
+
+		try {
+			this.overrideExtensions();
+		}
+		catch(e) {
+			dump(e+'\n');
+		}
 	},
 	_initialized : false,
 	applyPlatformDefaultPrefs : function TSTUtils_applyPlatformDefaultPrefs()
 	{
 		var OS = this.XULAppInfo.OS;
 		var processed = {};
-		this.getDescendant('extensions.treestyletab.platform.'+OS).forEach(function(aKey) {
-			var key = aKey.replace('platform.'+OS+'.', '');
-			this.setDefaultPref(key, this.getPref(aKey));
+		var originalKeys = this.getDescendant('extensions.treestyletab.platform.'+OS);
+		for (let i = 0, maxi = originalKeys.length; i < maxi; i++)
+		{
+			let originalKey = originalKeys[i];
+			let key = originalKey.replace('platform.'+OS+'.', '');
+			this.setDefaultPref(key, this.getPref(originalKey));
 			processed[key] = true;
-		}, this);
-		this.getDescendant('extensions.treestyletab.platform.default').forEach(function(aKey) {
-			var key = aKey.replace('platform.default.', '');
+		}
+		originalKeys = this.getDescendant('extensions.treestyletab.platform.default');
+		for (let i = 0, maxi = originalKeys.length; i < maxi; i++)
+		{
+			let originalKey = originalKeys[i];
+			let key = originalKey.replace('platform.default.', '');
 			if (!(key in processed))
-				this.setDefaultPref(key, this.getPref(aKey));
-		}, this);
+				this.setDefaultPref(key, this.getPref(originalKey));
+		}
+	},
+	kPREF_VERSION : 8,
+	migratePrefs : function TSTUtils_migratePrefs() 
+	{
+		// migrate old prefs
+		var orientalPrefs = [];
+		switch (this.getTreePref('prefsVersion'))
+		{
+			case 0:
+				orientalPrefs = orientalPrefs.concat([
+					'extensions.treestyletab.tabbar.fixed',
+					'extensions.treestyletab.enableSubtreeIndent',
+					'extensions.treestyletab.allowSubtreeCollapseExpand'
+				]);
+			case 1:
+			case 2:
+				if (this.getTreePref('urlbar.loadSameDomainToNewChildTab') !== null) {
+					let value = this.getTreePref('urlbar.loadSameDomainToNewChildTab');
+					this.setTreePref('urlbar.loadSameDomainToNewTab', value);
+					this.setTreePref('urlbar.loadSameDomainToNewTab.asChild', value);
+					if (value) this.setTreePref('urlbar.loadDifferentDomainToNewTab', value);
+					this.clearTreePref('urlbar.loadSameDomainToNewChildTab');
+				}
+			case 3:
+				if (this.getTreePref('loadDroppedLinkToNewChildTab') !== null) {
+					this.setTreePref('dropLinksOnTab.behavior',
+						this.getTreePref('loadDroppedLinkToNewChildTab.confirm') ?
+							this.kDROPLINK_ASK :
+						this.getTreePref('loadDroppedLinkToNewChildTab') ?
+							this.kDROPLINK_NEWTAB :
+							this.kDROPLINK_LOAD
+					);
+					this.clearTreePref('loadDroppedLinkToNewChildTab.confirm');
+					this.clearTreePref('loadDroppedLinkToNewChildTab');
+				}
+				if (this.getTreePref('openGroupBookmarkAsTabSubTree') !== null) {
+					let behavior = 0;
+					if (this.getTreePref('openGroupBookmarkAsTabSubTree.underParent'))
+						behavior += this.kGROUP_BOOKMARK_USE_DUMMY;
+					if (!this.getTreePref('openGroupBookmarkBehavior.confirm')) {
+						behavior += (
+							this.getTreePref('openGroupBookmarkAsTabSubTree') ?
+								this.kGROUP_BOOKMARK_SUBTREE :
+							this.getTreePref('browser.tabs.loadFolderAndReplace') ?
+								this.kGROUP_BOOKMARK_REPLACE :
+								this.kGROUP_BOOKMARK_SEPARATE
+						);
+					}
+					this.setTreePref('openGroupBookmark.behavior', behavior);
+					this.clearTreePref('openGroupBookmarkBehavior.confirm');
+					this.clearTreePref('openGroupBookmarkAsTabSubTree');
+					this.clearTreePref('openGroupBookmarkAsTabSubTree.underParent');
+					this.setPref('browser.tabs.loadFolderAndReplace', !!(behavior & this.kGROUP_BOOKMARK_REPLACE));
+				}
+			case 4:
+				let (prefs = [
+						'extensions.treestyletab.autoCollapseExpandSubTreeOnSelect',
+						'extensions.treestyletab.autoCollapseExpandSubTreeOnSelect.onCurrentTabRemove',
+						'extensions.treestyletab.autoCollapseExpandSubTreeOnSelect.whileFocusMovingByShortcut',
+						'extensions.treestyletab.autoExpandSubTreeOnAppendChild',
+						'extensions.treestyletab.autoExpandSubTreeOnCollapsedChildFocused',
+						'extensions.treestyletab.collapseExpandSubTree.dblclick',
+						'extensions.treestyletab.createSubTree.underParent',
+						'extensions.treestyletab.show.context-item-reloadTabSubTree',
+						'extensions.treestyletab.show.context-item-removeTabSubTree',
+						'extensions.treestyletab.show.context-item-bookmarkTabSubTree',
+						'extensions.multipletab.show.multipletab-selection-item-removeTabSubTree',
+						'extensions.multipletab.show.multipletab-selection-item-createSubTree'
+					]) {
+					for (let i = 0, maxi = prefs.length; i < maxi; i++)
+					{
+						let pref = prefs[i];
+						let value = this.getPref(pref);
+						if (value === null) continue;
+						this.setPref(pref.replace('SubTree', 'Subtree'), value);
+						this.clearPref(pref);
+					}
+				}
+			case 5:
+				let (behavior = this.getTreePref('openGroupBookmark.behavior')) {
+					behavior = behavior | 2048;
+					this.setTreePref('openGroupBookmark.behavior', behavior);
+				}
+			case 6:
+				let (
+					general = this.getTreePref('autoAttachNewTabsAsChildren'),
+					search = this.getTreePref('autoAttachSearchResultAsChildren')
+					) {
+					if (general !== null)
+						this.setTreePref('autoAttach', general);
+					if (search !== null)
+						this.setTreePref('autoAttach.searchResult', search);
+				}
+			case 7:
+				let (
+					enabled = this.getTreePref('autoCollapseExpandSubtreeOnSelect.whileFocusMovingByShortcut'),
+					delay = this.getTreePref('autoCollapseExpandSubtreeOnSelect.whileFocusMovingByShortcut.delay')
+					) {
+					if (enabled !== null) {
+						this.setTreePref('autoExpandSubtreeOnSelect.whileFocusMovingByShortcut', enabled);
+						this.setTreePref('autoExpandSubtreeOnSelect.whileFocusMovingByShortcut.collapseOthers', enabled);
+					}
+					if (delay !== null)
+						this.setTreePref('autoExpandSubtreeOnSelect.whileFocusMovingByShortcut.delay', delay);
+				}
+			default:
+				for (let i = 0, maxi = orientalPrefs.length; i < maxi; i++)
+				{
+					let pref = orientalPrefs[i];
+					let value = this.getPref(pref);
+					if (value === null) continue;
+					this.setPref(pref+'.horizontal', value);
+					this.setPref(pref+'.vertical', value);
+					this.clearPref(pref);
+				}
+				break;
+		}
+		this.setTreePref('prefsVersion', this.kPREF_VERSION);
 	},
 	
 	updateAeroPeek : function TSTUtils_updateAeroPeek() 
@@ -393,6 +538,24 @@ var TreeStyleTabUtils = {
 		this.AeroPeek = ns.AeroPeek;
 	},
  
+	overrideExtensions : function TSTUtils_overrideExtensions() 
+	{
+		// Scriptish
+		// https://addons.mozilla.org/firefox/addon/scriptish/
+		if (this.getTreePref('compatibility.Scriptish')) {
+			try {
+				let tabModule = Components.utils.import('resource://scriptish/utils/Scriptish_openInTab.js', {});
+				let Scriptish_openInTab = tabModule.Scriptish_openInTab;
+				tabModule.Scriptish_openInTab = function(aURL, aLoadInBackground, aReuse, aChromeWin) {
+					aChromeWin.TreeStyleTabService.readyToOpenChildTabNow(aChromeWin.gBrowser);
+					return Scriptish_openInTab.apply(this, arguments);
+				};
+			}
+			catch(e) {
+			}
+		}
+	},
+ 
 	updateNarrowScrollbarStyle : function TSTUtils_updateNarrowScrollbarStyle() 
 	{
 		const SSS = Cc['@mozilla.org/content/style-sheet-service;1']
@@ -434,7 +597,12 @@ var TreeStyleTabUtils = {
 					margin-left: 0;
 					margin-right: 0;
 				}
+
+				%FORCE_NARROW_SCROLLBAR%
 			]]>.toString()
+				.replace(/%FORCE_NARROW_SCROLLBAR%/g,
+					this.getTreePref('tabbar.narrowScrollbar.overrideSystemAppearance') ?
+						this.kOVERRIDE_SYSTEM_SCROLLBAR_APPEARANCE : '' )
 				.replace(/%MODE%/g, this.kMODE)
 				.replace(/%NARROW%/g, this.kNARROW_SCROLLBAR)
 				.replace(/%SIZE%/g, this.getTreePref('tabbar.narrowScrollbar.size'))
@@ -442,6 +610,17 @@ var TreeStyleTabUtils = {
 		this.lastAgentSheet = this.makeURIFromSpec(style);
 		SSS.loadAndRegisterSheet(this.lastAgentSheet, SSS.AGENT_SHEET);
 	},
+	kOVERRIDE_SYSTEM_SCROLLBAR_APPEARANCE : <![CDATA[
+		tabs.tabbrowser-tabs[%MODE%="vertical"][%NARROW%="true"]
+		  .tabbrowser-arrowscrollbox
+		  > scrollbox
+		  > scrollbar[orient="vertical"] {
+			appearance: none;
+			-moz-appearance: none;
+			background: ThreeDFace;
+			border: 1px solid ThreeDShadow;
+		}
+	]]>.toString(),
 	lastAgentSheet : null,
   
 	observe : function TSTUtils_observe(aSubject, aTopic, aData) 
@@ -638,27 +817,23 @@ var TreeStyleTabUtils = {
 		if (behavior & this.kUNDO_CLOSE_SET) behavior ^= this.kUNDO_CLOSE_SET;
 
 		var self = this;
-		var neverAskState = !(behavior & this.kUNDO_ASK);
-		var checkbox = {
-				label : this.treeBundle.getString('undoCloseTabSetBehavior.never'),
-				checked : neverAskState
-			};
-		return confirmWithTab({
-				tab      : aRestoredTab,
+		return confirmWithPopup({
+				browser  : aRestoredTab.linkedBrowser,
 				label    : this.treeBundle.getFormattedString('undoCloseTabSetBehavior.label', [aCount]),
 				value    : 'treestyletab-undo-close-tree',
+				image    : 'chrome://treestyletab/content/res/icon.png',
 				buttons  : [
-					this.treeBundle.getString('undoCloseTabSetBehavior.set'),
-					this.treeBundle.getString('undoCloseTabSetBehavior.separate')
+					this.treeBundle.getString('undoCloseTabSetBehavior.restoreOnce'),
+					this.treeBundle.getString('undoCloseTabSetBehavior.restoreForever'),
+					this.treeBundle.getString('undoCloseTabSetBehavior.ignoreForever')
 				],
-				checkbox : checkbox,
-				cancelEvents : ['TabClose', 'SSTabRestoring']
+				persistence : -1 // don't hide even if the tab is restored after the panel is shown.
 			})
 			.next(function(aButtonIndex) {
-				if (aButtonIndex == 0) {
+				if (aButtonIndex < 2) {
 					behavior |= self.kUNDO_CLOSE_SET;
 				}
-				if (checkbox.checked != neverAskState) {
+				if (aButtonIndex > 0) {
 					behavior ^= self.kUNDO_ASK;
 					self.setTreePref('undoCloseTabSet.behavior', behavior);
 				}
@@ -676,26 +851,28 @@ var TreeStyleTabUtils = {
 	doAndWaitDOMEvent : function TSTUtils_doAndWaitDOMEvent() 
 	{
 		var type, target, delay, task;
-		Array.slice(arguments).forEach(function(aArg) {
-			switch(typeof aArg)
+		for (let i = 0, maxi = arguments.length; i < maxi; i++)
+		{
+			let arg = arguments[i];
+			switch(typeof arg)
 			{
 				case 'string':
-					type = aArg;
-					break;
+					type = arg;
+					continue;
 
 				case 'number':
-					delay = aArg;
-					break;
+					delay = arg;
+					continue;
 
 				case 'function':
-					task = aArg;
-					break;
+					task = arg;
+					continue;
 
 				default:
-					target = aArg;
-					break;
+					target = arg;
+					continue;
 			}
-		});
+		}
 
 		if (!target || !type) {
 			if (task) task();
@@ -720,7 +897,7 @@ var TreeStyleTabUtils = {
 					target.removeEventListener(type, listener, false);
 					done = true;
 				}
-			});
+			}).error(this.defaultDeferredErrorHandler);
 
 		target.addEventListener(type, listener, false);
 
@@ -750,6 +927,25 @@ var TreeStyleTabUtils = {
 		return doc.documentElement;
 	},
  
+	assertBeforeDestruction : function TSTUtils_assertBeforeDestruction(aNotDestructed) 
+	{
+		if (aNotDestructed)
+			return;
+
+		var message = 'ERROR: accessed after destruction!';
+		var error = new Error(message);
+		dump(message+'\n'+error.stack+'\n');
+		throw error;
+	},
+ 
+	defaultDeferredErrorHandler : function TSTUtils_defaultDeferredErrorHandler(aError) 
+	{
+		if (aError.stack)
+			Components.utils.reportError(aError.message+'\n'+aError.stack);
+		else
+			Components.utils.reportError(aError);
+	},
+ 
 // event 
 	
 	isNewTabAction : function TSTUtils_isNewTabAction(aEvent) 
@@ -788,7 +984,16 @@ var TreeStyleTabUtils = {
 	isEventFiredOnClickable : function TSTUtils_isEventFiredOnClickable(aEvent) 
 	{
 		return this.evaluateXPath(
-				'ancestor-or-self::*[contains(" button toolbarbutton scrollbar popup menupopup panel tooltip splitter textbox ", concat(" ", local-name(), " "))]',
+				'ancestor-or-self::*[contains(" button toolbarbutton scrollbar nativescrollbar popup menupopup panel tooltip splitter textbox ", concat(" ", local-name(), " "))]',
+				aEvent.originalTarget,
+				Ci.nsIDOMXPathResult.BOOLEAN_TYPE
+			).booleanValue;
+	},
+ 
+	isEventFiredOnScrollbar : function TSTUtils_isEventFiredOnScrollbar(aEvent) 
+	{
+		return this.evaluateXPath(
+				'ancestor-or-self::*[local-name()="scrollbar" or local-name()="nativescrollbar"]',
 				aEvent.originalTarget,
 				Ci.nsIDOMXPathResult.BOOLEAN_TYPE
 			).booleanValue;
@@ -807,54 +1012,87 @@ var TreeStyleTabUtils = {
 			return false;
 
 		var box = twisty.boxObject;
-		var minX = box.screenX;
-		var minY = box.screenY;
-		var maxX = minX + box.width;
-		var maxY = minY + box.height;
-		var icon  = tab.ownerDocument.getAnonymousElementByAttribute(tab, 'class', 'tab-icon');
-		var iconBox = icon.boxObject;
+		var left = box.screenX;
+		var top = box.screenY;
+		var right = left + box.width;
+		var bottom = top + box.height;
+		var favicon = this.getFaviconRect(tab);
 		if (!box.width || !box.height) {
-			minX = iconBox.screenX;
-			minY = iconBox.screenY;
-			maxX = minX + iconBox.width;
-			maxY = minY + iconBox.height;
+			left = favicon.left;
+			top = favicon.top;
+			right = favicon.right;
+			bottom = favicon.bottom;
 		}
-		if (
-			box.width && box.height &&
+		else if (
 			this.shouldExpandTwistyArea &&
 			!this._expandTwistyAreaBlockers.length
 			) {
-			minX = Math.min(minX, iconBox.screenX);
-			minY = Math.min(minY, iconBox.screenY);
-			maxX = Math.max(maxX, iconBox.screenX + iconBox.width);
-			maxY = Math.max(maxY, iconBox.screenY + iconBox.height);
+			left = Math.min(left, favicon.left);
+			top = Math.min(top, favicon.top);
+			right = Math.max(right, favicon.right);
+			bottom = Math.max(bottom, favicon.bottom);
 		}
 
 		var x = aEvent.screenX;
 		var y = aEvent.screenY;
-		return (x >= minX && x <= maxX && y >= minY && y <= maxY);
+		return (x >= left && x <= right && y >= top && y <= bottom);
+	},
+	getFaviconRect : function TSTUtils_getFaviconRect(aTab)
+	{
+		var icon  = aTab.ownerDocument.getAnonymousElementByAttribute(aTab, 'class', 'tab-icon-image');
+		var iconBox = icon.boxObject;
+		var iconRect = {
+				left   : iconBox.screenX,
+				top    : iconBox.screenY,
+				right  : iconBox.screenX + iconBox.width,
+				bottom : iconBox.screenY + iconBox.height
+			};
+
+		var throbber  = aTab.ownerDocument.getAnonymousElementByAttribute(aTab, 'class', 'tab-throbber');
+		var throbberBox = throbber.boxObject;
+		var throbberRect = {
+				left   : throbberBox.screenX,
+				top    : throbberBox.screenY,
+				right  : throbberBox.screenX + throbberBox.width,
+				bottom : throbberBox.screenY + throbberBox.height
+			};
+
+		if (!iconBox.width && !iconBox.height)
+			return throbberRect;
+
+		if (!throbberBox.width && !throbberBox.height)
+			return iconRect;
+
+		return {
+			left   : Math.min(throbberRect.left, iconRect.left),
+			right  : Math.max(throbberRect.right, iconRect.right),
+			top    : Math.min(throbberRect.top, iconRect.top),
+			bottom : Math.max(throbberRect.bottom, iconRect.bottom)
+		};
 	},
 	
 	// called with target(nsIDOMEventTarget), document(nsIDOMDocument), type(string) and data(object) 
-	fireDataContainerEvent : function()
+	fireDataContainerEvent : function TSTUtils_fireDataContainerEvent()
 	{
 		var target, document, type, data, canBubble, cancellable;
-		Array.slice(arguments).forEach(function(aArg) {
-			if (typeof aArg == 'boolean') {
+		for (let i = 0, maxi = arguments.length; i < maxi; i++)
+		{
+			let arg = arguments[i];
+			if (typeof arg == 'boolean') {
 				if (canBubble === void(0))
-					canBubble = aArg;
+					canBubble = arg;
 				else
-					cancellable = aArg;
+					cancellable = arg;
 			}
-			else if (typeof aArg == 'string')
-				type = aArg;
-			else if (aArg instanceof Ci.nsIDOMDocument)
-				document = aArg;
-			else if (aArg instanceof Ci.nsIDOMEventTarget)
-				target = aArg;
+			else if (typeof arg == 'string')
+				type = arg;
+			else if (arg instanceof Ci.nsIDOMDocument)
+				document = arg;
+			else if (arg instanceof Ci.nsIDOMEventTarget)
+				target = arg;
 			else
-				data = aArg;
-		});
+				data = arg;
+		}
 		if (!target)
 			target = document;
 		if (!document)
@@ -862,12 +1100,13 @@ var TreeStyleTabUtils = {
 
 		var event = document.createEvent('DataContainerEvent');
 		event.initEvent(type, canBubble, cancellable);
-		for (var i in data)
+		var properties = Object.keys(data);
+		for (let i = 0, maxi = properties.length; i < maxi; i++)
 		{
-			if (!data.hasOwnProperty(i))
-				continue;
-			event.setData(i, data[i]);
-			event[i] = data[i]; // for backward compatibility
+			let property = properties[i];
+			let value = data[property];
+			event.setData(property, value);
+			event[property] = value; // for backward compatibility
 		}
 
 		return target.dispatchEvent(event);
@@ -1031,8 +1270,7 @@ var TreeStyleTabUtils = {
 	// workaround for http://piro.sakura.ne.jp/latest/blosxom/mozilla/extension/treestyletab/2009-09-29_debug.htm
 	checkCachedSessionDataExpiration : function TSTUtils_checkCachedSessionDataExpiration(aTab) 
 	{
-		var data = aTab.linkedBrowser.__SS_data || // Firefox 3.6-
-					aTab.linkedBrowser.parentNode.__SS_data; // -Frefox 3.5
+		var data = aTab.linkedBrowser.__SS_data;
 		if (data &&
 			data._tabStillLoading &&
 			aTab.getAttribute('busy') != 'true' &&
@@ -1044,17 +1282,19 @@ var TreeStyleTabUtils = {
 	{
 		if (!aTabs || aTabs.length <= 1) return;
 		var id = this.makeNewClosedSetId() + '::' + aTabs.length;
-		aTabs.forEach(function(aTab) {
-			this.setTabValue(aTab, this.kCLOSED_SET_ID, id);
-		}, this);
+		for (let i = 0, maxi = aTabs.length; i < maxi; i++)
+		{
+			this.setTabValue(aTabs[i], this.kCLOSED_SET_ID, id);
+		}
 	},
  
 	unmarkAsClosedSet : function TSTUtils_unmarkAsClosedSet(aTabs) /* PUBLIC API */ 
 	{
 		if (!aTabs || !aTabs.length) return;
-		aTabs.forEach(function(aTab) {
-			this.deleteTabValue(aTab, this.kCLOSED_SET_ID);
-		}, this);
+		for (let i = 0, maxi = aTabs.length; i < maxi; i++)
+		{
+			this.deleteTabValue(aTabs[i], this.kCLOSED_SET_ID);
+		}
 	},
  
 	useTMPSessionAPI : false, 
@@ -1169,6 +1409,18 @@ var TreeStyleTabUtils = {
 		return this.getTabFromChild(aEvent.originalTarget || aEvent.target);
 	},
  
+	getNewTabButtonFromEvent : function TSTUtils_getNewTabButtonFromEvent(aEvent) 
+	{
+		return this.evaluateXPath(
+				'ancestor-or-self::*['
+					+'@id="new-tab-button" or '
+					+'contains(concat(" ", normalize-space(@class), " "), " tabs-newtab-button ")'
+				+'][1]',
+				aEvent.originalTarget,
+				Ci.nsIDOMXPathResult.FIRST_ORDERED_NODE_TYPE
+			).singleNodeValue;
+	},
+ 
 	getSplitterFromEvent : function TSTUtils_getSplitterFromEvent(aEvent) 
 	{
 		return this.evaluateXPath(
@@ -1192,8 +1444,9 @@ var TreeStyleTabUtils = {
 		var b = aTabBrowser || this.browser;
 		var top = aFrame.top;
 		var tabs = this.getAllTabsArray(b);
-		for each (var tab in tabs)
+		for (let i = 0, maxi = tabs.length; i < maxi; i++)
 		{
+			let tab = tabs[i];
 			if (tab.linkedBrowser.contentWindow == top)
 				return tab;
 		}
@@ -1232,10 +1485,12 @@ var TreeStyleTabUtils = {
 	cleanUpTabsArray : function TSTUtils_cleanUpTabsArray(aTabs) 
 	{
 		var newTabs = [];
-		aTabs.forEach(function(aTab) {
-			if (!aTab.parentNode) return; // ignore removed tabs
-			if (newTabs.indexOf(aTab) < 0) newTabs.push(aTab);
-		});
+		for (let i = 0, maxi = aTabs.length; i < maxi; i++)
+		{
+			let tab = aTabs[i];
+			if (!tab || !tab.parentNode) continue; // ignore removed tabs
+			if (newTabs.indexOf(tab) < 0) newTabs.push(tab);
+		}
 		newTabs.sort(this.sortTabsByOrder);
 		return newTabs;
 	},
@@ -1267,17 +1522,19 @@ var TreeStyleTabUtils = {
 		var groups = [];
 
 		var group = [];
-		this.cleanUpTabsArray(aTabs)
-			.forEach(function(aTab) {
-				var parent = this.getParentTab(aTab);
-				if (!parent || group.indexOf(parent) < 0) {
-					if (group.length) groups.push(group);
-					group = [aTab];
-				}
-				else {
-					group.push(aTab);
-				}
-			}, this);
+		aTabs = this.cleanUpTabsArray(aTabs);
+		for (let i = 0, maxi = aTabs.length; i < maxi; i++)
+		{
+			let tab = aTabs[i];
+			let parent = this.getParentTab(tab);
+			if (!parent || group.indexOf(parent) < 0) {
+				if (group.length) groups.push(group);
+				group = [tab];
+			}
+			else {
+				group.push(tab);
+			}
+		}
 		if (group.length) groups.push(group);
 		return groups;
 	},
@@ -1295,13 +1552,13 @@ var TreeStyleTabUtils = {
 		if (aTabBrowserChild.localName == 'tabbrowser') // itself
 			return aTabBrowserChild;
 
-		if (aTabBrowserChild.tabbrowser) // tabs, Firefox 4.0 or later
+		if (aTabBrowserChild.tabbrowser) // tabs
 			return aTabBrowserChild.tabbrowser;
 
-		if (aTabBrowserChild.localName == 'toolbar') // tabs toolbar, Firefox 4.0 or later
+		if (aTabBrowserChild.localName == 'toolbar') // tabs toolbar
 			return aTabBrowserChild.getElementsByTagName('tabs')[0].tabbrowser;
 
-		// tab context menu on Firefox 4.0
+		// tab context menu
 		var popup = this.evaluateXPath(
 				'ancestor-or-self::xul:menupopup[@id="tabContextMenu"]',
 				aTabBrowserChild,
@@ -1390,6 +1647,7 @@ var TreeStyleTabUtils = {
 	getAllTabs : function TSTUtils_getTabs(aTabBrowserChild) /* OBSOLETE */ 
 	{
 		var b = this.getTabBrowserFromChild(aTabBrowserChild);
+		this.assertBeforeDestruction(b && b.mTabContainer);
 		return this.evaluateXPath(
 			'descendant::xul:tab',
 			b.mTabContainer
@@ -1403,6 +1661,7 @@ var TreeStyleTabUtils = {
 	getTabs : function TSTUtils_getTabs(aTabBrowserChild) /* OBSOLETE */ 
 	{
 		var b = this.getTabBrowserFromChild(aTabBrowserChild);
+		this.assertBeforeDestruction(b && b.mTabContainer);
 		return this.evaluateXPath(
 			'descendant::xul:tab[not(@hidden="true")]',
 			b.mTabContainer
@@ -1416,6 +1675,7 @@ var TreeStyleTabUtils = {
 	getAllTabsArray : function TSTUtils_getAllTabsArray(aTabBrowserChild) 
 	{
 		var b = this.getTabBrowserFromChild(aTabBrowserChild);
+		this.assertBeforeDestruction(b && b.mTabContainer);
 		return Array.slice(b.mTabContainer.childNodes) ;
 	},
  
@@ -1426,6 +1686,7 @@ var TreeStyleTabUtils = {
 	getTabsArray : function TSTUtils_getTabsArray(aTabBrowserChild) 
 	{
 		var b = this.getTabBrowserFromChild(aTabBrowserChild);
+		this.assertBeforeDestruction(b && b.mTabContainer);
 		return b.visibleTabs || Array.slice(b.mTabContainer.childNodes) ;
 	},
  
@@ -1435,6 +1696,7 @@ var TreeStyleTabUtils = {
 	getFirstTab : function TSTUtils_getFirstTab(aTabBrowserChild) 
 	{
 		var b = this.getTabBrowserFromChild(aTabBrowserChild);
+		this.assertBeforeDestruction(b && b.mTabContainer);
 		var tabs = b.visibleTabs;
 		return tabs ? tabs[0] : b.mTabContainer.firstChild;
 	},
@@ -1445,9 +1707,11 @@ var TreeStyleTabUtils = {
 	getFirstNormalTab : function TSTUtils_getFirstNormalTab(aTabBrowserChild) 
 	{
 		var b = this.getTabBrowserFromChild(aTabBrowserChild);
+		this.assertBeforeDestruction(b && b.mTabContainer);
 		return this.evaluateXPath(
 			'descendant::xul:tab[not(@pinned="true") and not(@hidden="true")]',
 			b.mTabContainer,
+
 			Ci.nsIDOMXPathResult.FIRST_ORDERED_NODE_TYPE
 		).singleNodeValue;
 	},
@@ -1458,6 +1722,7 @@ var TreeStyleTabUtils = {
 	getLastTab : function TSTUtils_getLastTab(aTabBrowserChild) 
 	{
 		var b = this.getTabBrowserFromChild(aTabBrowserChild);
+		this.assertBeforeDestruction(b && b.mTabContainer);
 		var tabs = b.visibleTabs;
 		return tabs ? tabs[tabs.length-1] : b.mTabContainer.lastChild ;
 	},
@@ -1469,6 +1734,7 @@ var TreeStyleTabUtils = {
 	{
 		if (!aTab) return null;
 		var b = this.getTabBrowserFromChild(aTab);
+		this.assertBeforeDestruction(b && b.mTabContainer);
 		var tabs = b.visibleTabs;
 		if (tabs) {
 			let index = tabs.indexOf(aTab);
@@ -1486,6 +1752,7 @@ var TreeStyleTabUtils = {
 	{
 		if (!aTab) return null;
 		var b = this.getTabBrowserFromChild(aTab);
+		this.assertBeforeDestruction(b && b.mTabContainer);
 		var tabs = b.visibleTabs;
 		if (tabs) {
 			let index = tabs.indexOf(aTab);
@@ -1640,7 +1907,7 @@ var TreeStyleTabUtils = {
 			let self = this;
 			this.Deferred.next(function() {
 				self.stopToOpenChildTab(aFrameOrTabBrowser);
-			});
+			}).error(this.defaultDeferredErrorHandler);
 			return true;
 		}
 		return false;
@@ -1691,7 +1958,7 @@ var TreeStyleTabUtils = {
 			let self = this;
 			this.Deferred.next(function() {
 				self.stopToOpenChildTab(aFrameOrTabBrowser);
-			});
+			}).error(this.defaultDeferredErrorHandler);
 			return true;
 		}
 		return false;
@@ -1722,11 +1989,12 @@ var TreeStyleTabUtils = {
 	 */
 	readyToOpenNewTabGroupNow : function TSTUtils_readyToOpenNewTabGroupNow(aFrameOrTabBrowser) /* PUBLIC API */
 	{
+
 		if (this.readyToOpenNewTabGroup.apply(this, arguments)) {
 			let self = this;
 			this.Deferred.next(function() {
 				self.stopToOpenChildTab(aFrameOrTabBrowser);
-			});
+			}).error(this.defaultDeferredErrorHandler);
 			return true;
 		}
 		return false;
@@ -1854,27 +2122,53 @@ var TreeStyleTabUtils = {
 	{
 		if (!aTab) return null;
 
+		var parent;
 		if (this.tabsHash) { // XPath-less implementation
-			let parent = this.getTabById(aTab.getAttribute(this.kPARENT));
-			return (parent && parent != aTab) ? parent : null ;
+			parent = this.getTabById(aTab.getAttribute(this.kPARENT));
 		}
-
-		return this.evaluateXPath(
-			'preceding-sibling::xul:tab[@'+this.kID+'="'+aTab.getAttribute(this.kPARENT)+'"][1]',
-			aTab,
-			Ci.nsIDOMXPathResult.FIRST_ORDERED_NODE_TYPE
-		).singleNodeValue;
+		else {
+			parent =  this.evaluateXPath(
+				'preceding-sibling::xul:tab[@'+this.kID+'="'+aTab.getAttribute(this.kPARENT)+'"][1]',
+				aTab,
+				Ci.nsIDOMXPathResult.FIRST_ORDERED_NODE_TYPE
+			).singleNodeValue;
+		}
+		return (parent && parent != aTab) ? parent : null ;
 	},
  
 	getAncestorTabs : function TSTUtils_getAncestorTabs(aTab) /* PUBLIC API */ 
 	{
-		var tabs = [];
+		var tabs = [aTab];
 		var parentTab = aTab;
 		while (parentTab = this.getParentTab(parentTab))
 		{
+			if (tabs.indexOf(parentTab) > -1) {
+				let message = 'recursive tree detected!\n'+
+					tabs.concat([parentTab])
+					.reverse().map(function(aTab) {
+						return '  '+aTab._tPos+' : '+
+								aTab.label+'\n     '+
+								aTab.getAttribute(this.kID);
+					}, this).join('\n');
+				dump(message+'\n');
+				break;
+			}
+
+			if (aTab._tPos < parentTab._tPos) {
+				let message = 'broken tree detected!\n'+
+					tabs.concat([parentTab])
+					.reverse().map(function(aTab) {
+						return '  '+aTab._tPos+' : '+
+								aTab.label+'\n     '+
+								aTab.getAttribute(this.kID);
+					}, this).join('\n');
+				dump(message+'\n');
+			}
+
 			tabs.push(parentTab);
+			aTab = parentTab;
 		}
-		return tabs;
+		return tabs.slice(1);
 	},
  
 	getRootTab : function TSTUtils_getRootTab(aTab) /* PUBLIC API */ 
@@ -1882,13 +2176,8 @@ var TreeStyleTabUtils = {
 		if (!aTab) return null;
 
 		if (this.tabsHash) { // XPath-less implementation
-			let parent = aTab;
-			let root   = aTab;
-			while (parent = this.getParentTab(parent))
-			{
-				root = parent;
-			}
-			return root;
+			let ancestors = this.getAncestorTabs(aTab);
+			return ancestors.length ? ancestors[ancestors.length-1] : aTab ;
 		}
 
 		return this.evaluateXPath(
@@ -1918,7 +2207,8 @@ var TreeStyleTabUtils = {
 
 			let children = parentTab.getAttribute(this.kCHILDREN);
 			if (children) {
-				let list = ('|'+children).split('|'+aTab.getAttribute(this.kID))[1].split('|');
+				let list = ('|'+children).split('|'+aTab.getAttribute(this.kID));
+				list = list.length > 1 ? list[1].split('|') : [] ;
 				for (let i = 0, maxi = list.length; i < maxi; i++)
 				{
 					let firstChild = this.getTabById(list[i], aTab);
@@ -1978,7 +2268,7 @@ var TreeStyleTabUtils = {
 		).singleNodeValue;
 	},
  
-	getSiblingTabs : function TSTUtils_getSiblingTabs(aTab) /* PUBLIC API */
+	getSiblingTabs : function TSTUtils_getSiblingTabs(aTab) /* PUBLIC API */ 
 	{
 		var parent = this.getParentTab(aTab);
 
@@ -2104,47 +2394,38 @@ var TreeStyleTabUtils = {
  
 	getChildIndex : function TSTUtils_getChildIndex(aTab, aParent) /* PUBLIC API */ 
 	{
-		if (this.tabsHash) { // XPath-less implementation
-			let parent = this.getParentTab(aTab);
-			if (!aParent || !parent || aParent != parent) {
-				parent = aTab;
-				while (parent && parent != aParent)
-				{
-					aTab = parent;
-					parent = this.getParentTab(parent);
-				}
-				if (parent != aParent)
-					return -1;
-				aParent = parent;
+		var parent = this.getParentTab(aTab);
+		if (!aParent || !parent || aParent != parent) {
+			let tabs = [aTab].concat(this.getAncestorTabs(aTab));
+			parent = aTab;
+			for (let i = 0, maxi = tabs.length; i < maxi && parent != aParent; i++)
+			{
+				aTab = parent;
+				parent = i < maxi ? tabs[i+1] : null ;
 			}
-
-			if (aParent) {
-				let children = aParent.getAttribute(this.kCHILDREN);
-				let list = children.split('|');
-				let id = aTab.getAttribute(this.kID);
-				for (let i = 0, maxi = list.length; i < maxi; i++)
-				{
-					if (list[i] == id) return i;
-				}
+			if (parent != aParent)
 				return -1;
-			}
-			else {
-				let tabs = this.rootTabs;
-				for (let i = 0, maxi = tabs.length; i < maxi; i++)
-				{
-					if (tabs[i] == aTab) return i;
-				}
+			aParent = parent;
+		}
+
+		if (aParent) {
+			let children = aParent.getAttribute(this.kCHILDREN);
+			let list = children.split('|');
+			let id = aTab.getAttribute(this.kID);
+			for (let i = 0, maxi = list.length; i < maxi; i++)
+			{
+				if (list[i] == id) return i;
 			}
 			return -1;
 		}
-
-		var parent = aTab.getAttribute(this.kPARENT);
-		if (!parent) return -1;
-		return this.evaluateXPath(
-			'count(preceding-sibling::xul:tab[@'+this.kPARENT+' and @'+this.kPARENT+'="'+parent+'"])',
-			aTab,
-			Ci.nsIDOMXPathResult.NUMBER_TYPE
-		).numberValue;
+		else {
+			let tabs = this.rootTabs;
+			for (let i = 0, maxi = tabs.length; i < maxi; i++)
+			{
+				if (tabs[i] == aTab) return i;
+			}
+		}
+		return -1;
 	},
  
 	getXOffsetOfTab : function TSTUtils_getXOffsetOfTab(aTab) 
@@ -2173,6 +2454,20 @@ var TreeStyleTabUtils = {
 			Ci.nsIDOMXPathResult.NUMBER_TYPE
 		).numberValue;
 	},
+	getFutureBoxObject : function TSTUtils_getFutureBoxObject(aTab)
+	{
+		var tabBox = aTab.boxObject;
+		var xOffset = this.getXOffsetOfTab(aTab);
+		var yOffset = this.getYOffsetOfTab(aTab);
+		return {
+			width     : tabBox.width,
+			height    : tabBox.height,
+			x         : tabBox.x + xOffset,
+			y         : tabBox.y + yOffset,
+			screenX   : tabBox.screenX + xOffset,
+			screenY   : tabBox.screenY + yOffset
+		};
+	},
  
 	isGroupTab : function TSTUtils_isGroupTab(aTab, aLazyCheck) 
 	{
@@ -2191,15 +2486,17 @@ var TreeStyleTabUtils = {
 				).numberValue;
 	},
  
-	forceExpandTabs : function TSTUtils_forceExpandTabs(aTabs)
+	forceExpandTabs : function TSTUtils_forceExpandTabs(aTabs) 
 	{
 		var collapsedStates = aTabs.map(function(aTab) {
 				return this.getTabValue(aTab, this.kSUBTREE_COLLAPSED) == 'true';
 			}, this);
-		aTabs.forEach(function(aTab) {
-			this.collapseExpandSubtree(aTab, false, true);
-			this.collapseExpandTab(aTab, false, true);
-		}, this);
+		for (let i = 0, maxi = aTabs.length; i < maxi; i++)
+		{
+			let tab = aTabs[i];
+			this.collapseExpandSubtree(tab, false, true);
+			this.collapseExpandTab(tab, false, true);
+		}
 		return collapsedStates;
 	},
  
@@ -2263,30 +2560,32 @@ var TreeStyleTabUtils = {
 		while (aExpandStates.length < aTabs.length) aExpandStates.push(-1);
 
 		var parentTab = null;
-		aTabs.forEach(function(aTab, aIndex) {
-			if (sv.isCollapsed(aTab)) sv.collapseExpandTab(aTab, false, true);
-			sv.partTab(aTab);
+		for (let i = 0, maxi = aTabs.length; i < maxi; i++)
+		{
+			let tab = aTabs[i];
+			if (sv.isCollapsed(tab)) sv.collapseExpandTab(tab, false, true);
+			sv.detachTab(tab);
 
-			var parentIndexInTree = aTreeStructure[aIndex];
+			let parentIndexInTree = aTreeStructure[i];
 			if (parentIndexInTree < 0) // there is no parent, so this is a new parent!
-				parentTab = aTab.getAttribute(sv.kID);
+				parentTab = tab.getAttribute(sv.kID);
 
-			var parent = sv.getTabById(parentTab);
+			let parent = sv.getTabById(parentTab);
 			if (parent) {
 				let tabs = [parent].concat(sv.getDescendantTabs(parent));
 				parent = parentIndexInTree < tabs.length ? tabs[parentIndexInTree] : parent ;
 			}
 			if (parent) {
-				sv.attachTabTo(aTab, parent, {
-					dontExpand : true,
-					dontMove   : true
+				sv.attachTabTo(tab, parent, {
+					forceExpand : true,
+					dontMove    : true
 				});
 			}
-		});
+		}
 
 		for (let i = aTabs.length-1; i > -1; i--)
 		{
-			sv.collapseExpandSubtree(aTabs[i], !aExpandStates[i], true);
+			sv.collapseExpandSubtree(aTabs[i], !sv.hasChildTabs(aTabs[i]) || !aExpandStates[i], true);
 		}
 	},
  
@@ -2409,6 +2708,12 @@ var TreeStyleTabUtils = {
 			case 'extensions.treestyletab.twisty.expandSensitiveArea':
 				return this.shouldExpandTwistyArea = value;
 
+			case 'extensions.treestyletab.counter.role.horizontal':
+				return this.counterRoleHorizontal = value;
+
+			case 'extensions.treestyletab.counter.role.vertical':
+				return this.counterRoleVertical = value;
+
 			default:
 				return;
 		}
@@ -2480,7 +2785,5 @@ var TreeStyleTabUtils = {
   
 }; 
  
-// rap('end of definition of utils');
 TreeStyleTabUtils.init(); 
-// rap('end of initialization of utils');
   
